open F

(* A type-checker for System F. *)

type type_error =
  | TypeMismatch of debruijn_type * debruijn_type
  | ArityMismatch of arity_requirement * debruijn_type
  | NotAnArrow of debruijn_type
  | NotAProduct of debruijn_type
  | NotAForall of debruijn_type
  | UnboundTermVariable of tevar
  | TwoOccurencesOfSameVariable of string

(* An arity-related requirement on a certain (sum or product) type
   arising during type-checking:
   - [Index i]: [i] should be a valid index within the type
   - [Total n]: the type should have [n] components.
*)
and arity_requirement = Index of int | Total of int

exception TypeError of type_error

(* [typeof env t] type-checks the closed term [t] in datatype environment [env]
   and constructs its type. *)
val typeof: debruijn_datatype_env -> debruijn_term -> debruijn_type

